@import "basic.css";

/*Color main components with a dark theme*/
div, span, code {
    background-color: #181a1b !important;
}

.wy-side-nav-search {
    background-color: #005fff !important;
}

.wy-side-nav-search div {
    background-color: #005fff !important;
}

/*Font color is mainly white*/
.rst-content p, li, h1, h2, h3, h4, h5, h6, .highlight-console, .n, .section {
    color: #FFFFFF;
}

/*The side menu is slightly more grey and lighter than the overall theme*/
.wy-menu, .wy-menu-vertical {
    background-color: #2D2E2F !important;
}

.wy-side-scroll {
    background-color: #2D2E2F !important;
}

.wy-menu .caption-text {
    background-color: #2D2E2F !important;
}

.caption-text {
    background-color: #181a1b !important;
}

.figure p {
    background-color: #181a1b !important;
}

/*Toctree wrapper on index page has a dark background unlike the other menu items*/
.toctree-wrapper .compound ul .toctree-l1, .toctree-wrapper .compound ul .toctree-l2 {
    background-color: #181a1b !important;
    color: #005fff !important;
}

/*The current menu section has a blue background*/
.wy-menu-vertical li.toctree-l1.current li.toctree-l2 > a:hover {
    color: #FFFFFF !important;
    background: #005fff !important;
}

/*Subitems under the current section are displayed in grey again*/
.wy-menu-vertical li.toctree-l1.current li.toctree-l2 > a {
    color: #FFFFFF !important;
    background: #2D2E2F !important;
}

/*Current hovered item has a blue background*/
.wy-menu-vertical li.toctree-l2.current li.toctree-l3 > a:hover {
    color: #FFFFFF !important;
    background: #005fff !important;
}

.wy-menu-vertical li.toctree-l3.current li.toctree-l4 > a:hover {
    color: #FFFFFF !important;
    background: #005fff !important;
}

.wy-menu-vertical li.toctree-l2.current > a:hover {
    color: #FFFFFF !important;
    background: #005fff !important;
}

/* Child items of current expanded parent section are white font on dark background*/

.wy-menu-vertical li.toctree-l2.current li.toctree-l3 > a {
    color: #FFFFFF !important;
    background: #2D2E2F !important;
}

.wy-menu-vertical li.toctree-l3.current li.toctree-l4 > a {
    color: #FFFFFF !important;
    background: #2D2E2F !important;
}

.wy-menu-vertical li.toctree-l1.current {
    color: #FFFFFF !important;
    background: #005fff !important;
    border-color: #005fff !important;
}

.wy-menu-vertical li.toctree-l1.current > a {
    color: #FFFFFF !important;
    background: #005fff !important;
    border-color: #005fff !important;
}

.wy-menu-vertical li.toctree-l1.current > a:hover {
    color: #FFFFFF !important;
    background: #005fff !important;
    border-color: #005fff !important;
}

/*The expand toctree items have the same background as its corresponding section*/
.wy-menu-vertical li.toctree-l1 a span.toctree-expand {
    background-color: #005fff !important;
}

.wy-menu-vertical li.toctree-l2 a span.toctree-expand {
    background-color: #2D2E2F !important;
}

.wy-menu-vertical li.toctree-l2 a:hover span.toctree-expand {
    background-color: #005fff !important;
}

.wy-menu-vertical li.toctree-l3 a span.toctree-expand {
    background-color: #2D2E2F !important;
}

.wy-menu-vertical li.toctree-l3 a:hover span.toctree-expand {
    background-color: #005fff !important;
}

.code .docutils .literal .notranslate .pre {
    background-color: #181a1b !important;
}

/*Color footer separately corresponding to overall dark theme*/
footer {
    color: #FFFFFF;
}

/*Color footer buttons in blue*/
.rst-footer-buttons a, .rst-footer-buttons a:hover, .rst-footer-buttons span {
    background-color: #005fff !important;
}

.wy-side-nav-search a {
    background-color: #005fff !important;
}

.version {
    background-color: #005fff !important;
    color: #FFFFFF !important;
}

/*Set max width to none so the theme uses all available width*/
.wy-nav-content {
    max-width: none;
}
